package types

const (
	// DefaultAPIVersion is the version of the API the server defaults to.
	DefaultAPIVersion = "1.40" // See https://docs.docker.com/engine/api/v1.40/

	// DefaultAPIVersion is the minimal required version of the API.
	MinimalAPIVersion = "1.24"
)
